@@ -1,12 +1,13 @@
 import logging
 import re
+from copy import deepcopy
 from datetime import datetime
 from itertools import groupby
 from decimal import Decimal
 
 from PySide6.QtWidgets import QApplication
-from jal.constants import PredefinedCategory
-from jal.widgets.helpers import ManipulateDate
+from jal.constants import PredefinedCategory, PredefinedAsset
+from jal.widgets.helpers import ManipulateDate, ts2dt
 from jal.db.account import JalAccount
 from jal.db.asset import JalAsset
 from jal.db.operations import Dividend
@@ -47,7 +48,8 @@ class IBKR_AssetType:
         try:
             self.type = self._asset_types[asset_type]
         except KeyError:
-            logging.warning(QApplication.translate("IBKR", "Asset type isn't supported: ") + f"'{asset_type}'")
+            raise Statement_ImportError(
+                QApplication.translate("IBKR", "Asset type isn't supported: ") + f"'{asset_type}'")
         if self.type == FOF.ASSET_STOCK and subtype:  # distinguish ADR and ETF from stocks
             try:
                 self.type = self._asset_types[subtype]
@@ -77,7 +79,8 @@ class IBKR_CorpActionType:
         try:
             self.type = self._corporate_action_types[action_type]
         except KeyError:
-            logging.warning(QApplication.translate("IBKR", "Corporate action isn't supported: ") + f"{action_type}")
+            raise Statement_ImportError(
+                QApplication.translate("IBKR", "Corporate action isn't supported: ") + f"{action_type}")
 
 
 # -----------------------------------------------------------------------------------------------------------------------
@@ -121,6 +124,8 @@ class StatementIBKR(StatementXML):
     statement_tag = 'FlexStatement'
     level_tag = 'levelOfDetail'
     CancelledFlag = 'Ca'
+    ReversalSuffix = " - REVERSAL"
+    CancelPrefix = "CANCEL "
 
     def __init__(self):
         super().__init__()
@@ -214,6 +219,7 @@ class StatementIBKR(StatementXML):
                                             ('symbol', 'asset', IBKR_Asset, 0),
                                             ('currency', 'currency', IBKR_Currency, None),
                                             ('dateTime', 'timestamp', datetime, None),
+                                            ('reportDate', 'reported', datetime, None),
                                             ('amount', 'amount', float, None),
                                             ('tradeID', 'number', str, ''),
                                             ('description', 'description', str, None)],
@@ -450,7 +456,8 @@ class StatementIBKR(StatementXML):
                 if len(trade) == 1:
                     trade[0]['note'] = description
                 else:
-                    logging.warning(self.tr("Original trade not found for Option E&A&E operation: ") + f"{option}")
+                    raise Statement_ImportError(
+                        self.tr("Original trade not found for Option E&A&E operation: ") + f"{option}")
                 cnt += 1
         logging.info(self.tr("Options E&A&E loaded: ") + f"{cnt} ({len(options)})")
 
@@ -736,11 +743,12 @@ class StatementIBKR(StatementXML):
     def load_cash_transactions(self, cash):
         cnt = 0
         dividends = list(filter(lambda tr: tr['type'] in ['Dividends', 'Payment In Lieu Of Dividends'], cash))
+        dividends = self.aggregate_dividends(dividends)
         asset_payments_base = max([0] + [x['id'] for x in self._data[FOF.ASSET_PAYMENTS]]) + 1
         for i, dividend in enumerate(dividends):
             dividend['id'] = asset_payments_base + i
             dividend['type'] = FOF.PAYMENT_DIVIDEND
-            self.drop_extra_fields(dividend, ["currency"])
+            self.drop_extra_fields(dividend, ["currency", "reported"])
             self._data[FOF.ASSET_PAYMENTS].append(dividend)
             cnt += 1
         asset_payments_base += cnt
@@ -748,11 +756,12 @@ class StatementIBKR(StatementXML):
         for i, bond_interest in enumerate(bond_interests):
             bond_interest['id'] = asset_payments_base + i
             bond_interest['type'] = FOF.PAYMENT_INTEREST
-            self.drop_extra_fields(bond_interest, ["currency"])
+            self.drop_extra_fields(bond_interest, ["currency", "reported"])
             self._data[FOF.ASSET_PAYMENTS].append(bond_interest)
             cnt += 1
 
         taxes = list(filter(lambda tr: tr['type'] == 'Withholding Tax', cash))
+        taxes = self.aggregate_taxes(taxes)
         for tax in taxes:
             cnt += self.apply_tax_withheld(tax)
 
@@ -768,7 +777,7 @@ class StatementIBKR(StatementXML):
                 transfer['account'] = [transfer['account'], 0, 0]
                 transfer['withdrawal'] = transfer['deposit'] = -transfer['amount']
             transfer['fee'] = 0.0
-            self.drop_extra_fields(transfer, ["type", "amount", "currency"])
+            self.drop_extra_fields(transfer, ["type", "amount", "currency", "reported"])
             self._data[FOF.TRANSFERS].append(transfer)
             cnt += 1
 
@@ -785,12 +794,68 @@ class StatementIBKR(StatementXML):
             else:
                 category = -PredefinedCategory.Fees
             fee['lines'] = [{'amount': fee['amount'], 'category': category, 'description': fee['description']}]
-            self.drop_extra_fields(fee, ["type", "amount", "description", "asset", "number", "currency"])
+            self.drop_extra_fields(fee, ["type", "amount", "description", "asset", "number", "currency", "reported"])
             self._data[FOF.INCOME_SPENDING].append(fee)
             cnt += 1
 
         logging.info(self.tr("Cash transactions loaded: ") + f"{cnt} ({len(cash)})")
 
+    # Method takes a list of dividend dictionaries and checks for REVERSAL and CANCEL
+    # For such description it looks for matching record (for the same symbol) with opposite amount and the same payment
+    # and report dates. Description may be different!
+    # FIXME - this algo may be not fully correct for CANCEL as it may cancel older record
+    def aggregate_dividends(self, dividends: list) -> list:
+        is_reversal = lambda x: self.ReversalSuffix in x or x.startswith(self.CancelPrefix)
+        payments = [x for x in deepcopy(dividends) if not is_reversal(x['description'])]
+        reversals = [x for x in deepcopy(dividends) if is_reversal(x['description'])]
+        # Drop reversals that match payments exactly
+        for reversal in reversals:
+            t_payment = deepcopy(reversal)  # target payment to search for
+            t_payment['description'] = t_payment['description'].replace(self.ReversalSuffix, '')
+            t_payment['description'] = t_payment['description'].replace(self.CancelPrefix, '')
+            t_payment['amount'] = -t_payment['amount']
+            if t_payment not in payments:
+                no_description = lambda x: {i:x[i] for i in x if i != 'description'}
+                m_payments = [x for x in payments if no_description(x) == no_description(t_payment)]
+                if len(m_payments):
+                    payments.remove(m_payments[0])
+                    logging.warning(self.tr("Payment was reversed by approximate description: ") +
+                             f"{ts2dt(t_payment['timestamp'])}, '{t_payment['description']}': {t_payment['amount']}")
+                else:
+                    raise Statement_ImportError(self.tr("Can't find match for reversal: ") + f"{reversal}")
+            else:
+                payments.remove(t_payment) # Source payment found -> remove it
+                logging.info(self.tr("Payment was reversed: ") +
+                             f"{ts2dt(t_payment['timestamp'])}, '{t_payment['description']}': {t_payment['amount']}")
+        return payments
+
+    # Method takes a list of taxes and checks if we have the same amount added and deducted the same day
+    # First it tries to find exact match. Second it does it again ignoring reportDate.
+    def aggregate_taxes(self, taxes: list) -> list:
+        payments = [x for x in deepcopy(taxes) if x['amount'] < 0]
+        reversals = [x for x in deepcopy(taxes) if x['amount'] > 0]
+        not_matched_reversals = []
+        for reversal in reversals:
+            t_payment = deepcopy(reversal)   # target payment to search for
+            t_payment['description'] = t_payment['description'].replace(self.CancelPrefix, '')
+            t_payment['amount'] = -t_payment['amount']
+            if t_payment not in payments:
+                no_report = lambda x: {i: x[i] for i in x if i != 'reported'}
+                m_payments = [x for x in payments if no_report(x) == no_report(t_payment)]
+                if len(m_payments):
+                    payments.remove(m_payments[0])
+                else:
+                    no_d_and_r = lambda x: {i: x[i] for i in x if i != 'description' and i != 'reported'}
+                    m_payments = [x for x in payments if no_d_and_r(x) == no_d_and_r(t_payment)]
+                    if len(m_payments):
+                        payments.remove(m_payments[0])
+                    else:
+                        not_matched_reversals.append(reversal)
+            else:
+                payments.remove(t_payment)    # it is possible to kill exact match silently
+        taxes = [x for x in taxes if x in payments or x in not_matched_reversals]
+        return taxes
+
     def load_taxes(self, taxes):
         cnt = 0   #FIXME Link this tax with asset
         tax_base = max([0] + [x['id'] for x in self._data[FOF.INCOME_SPENDING]]) + 1
@@ -823,8 +888,7 @@ class StatementIBKR(StatementXML):
 
         dividend = self.find_dividend4tax(tax['timestamp'], tax['account'], tax['asset'], previous_tax, new_tax, description)
         if dividend is None:
-            logging.warning(self.tr("Dividend not found for withholding tax: ") + f"{tax}, {previous_tax}")
-            return 0
+            raise Statement_ImportError(self.tr("Dividend not found for withholding tax: ") + f"{tax}, {previous_tax}")
         dividend["tax"] = new_tax
         # append new dividend if it came from DB and haven't been loaded in self._data yet
         if len([1 for x in self._data[FOF.ASSET_PAYMENTS] if x['id'] == dividend['id']]) == 0:
@@ -846,8 +910,10 @@ class StatementIBKR(StatementXML):
                      (x['type'] == FOF.PAYMENT_DIVIDEND or x['type'] == FOF.PAYMENT_STOCK_DIVIDEND)
                      and x['asset'] == asset_id and x['account'] == account_id]
         account = [x for x in self._data[FOF.ACCOUNTS] if x["id"] == account_id][0]
-        db_account = JalAccount(data=account, search=True, create=False).id()
-        asset = [x for x in self._data[FOF.ASSETS] if x["id"] == asset_id][0]
+        currency_symbol = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == account['currency']][0]['symbol']
+        db_currency = JalAsset(data={'symbol': currency_symbol, 'type': PredefinedAsset.Money}, search=True, create=False).id()
+        db_account = JalAccount(data={'number': account['number'], 'currency': db_currency}, search=True, create=False).id()
+        asset = self._asset(asset_id)
         isin = asset['isin'] if 'isin' in asset else ''
         symbols = [x for x in self._data[FOF.SYMBOLS] if x["asset"] == asset_id]
         db_asset = JalAsset(data={'isin': isin, 'symbol': symbols[0]['symbol']}, search=True, create=False).id()
