@@ -55,7 +55,7 @@ class BalancesModel(QAbstractTableModel):
                 return self._data[row]['account_name']
             else:
                 return PredefindedAccountType().get_name(self._data[row]['account_type'], default=self.tr("Total"))
-        elif column == 1:
+        elif column == 1:  # FIXME - should floating formatting be done by FloatDelegate? for columns 1 & 3
             return f"{self._data[row]['balance']:,.2f}" if self._data[row]['balance'] != 0 else ''
         elif column == 2:
             return self._data[row]['currency_name'] if self._data[row]['balance'] != 0 else ''
@@ -150,7 +150,7 @@ class BalancesModel(QAbstractTableModel):
         balances = sorted(balances, key=lambda x: (x['account_type']))
         self._data = []
         field_names = ["account_type", "account", "account_name", "currency", "currency_name", "balance", "balance_a",
-                       "unreconciled", "active", "level"]   # TODO It is better to take names from 'balances' dict
+                       "unreconciled", "active", "level"]
         current_type = 0
         for values in balances:
             values['level'] = 0
