@@ -67,7 +67,7 @@ class JalDB:
         if parse_version(sqlite_version) < parse_version(Setup.SQLITE_MIN_VERSION):
             db.close()
             return JalDBError(JalDBError.OutdatedSqlite)
-        JalDB._tables = db.tables(QSql.Tables)
+        JalDB._tables = db.tables(QSql.Tables) + db.tables(QSql.Views)  # Bitwise or somehow doesn't work here :(
         if not JalDB._tables:
             logging.info("Loading DB initialization script")
             error = self.run_sql_script(db_path + Setup.INIT_SCRIPT_PATH)
