@@ -6,7 +6,7 @@ from jal.db.db import JalDB
 import jal.db.account
 from jal.db.asset import JalAsset
 from jal.db.settings import JalSettings
-from jal.widgets.helpers import ts2str
+from jal.widgets.helpers import ts2dt
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -223,6 +223,9 @@ class LedgerTransaction:
     def number(self):
         return self._number
 
+    def amount(self):
+        return 0
+
     def description(self) -> str:
         return ''
 
@@ -334,6 +337,9 @@ class IncomeSpending(LedgerTransaction):
     def lines(self) -> list:
         return self._details
 
+    def amount(self) -> Decimal:
+        return self._amount
+
     def processLedger(self, ledger):
         if len(self._details) == 0:
             self.dump()
@@ -411,7 +417,8 @@ class Dividend(LedgerTransaction):
         dividends = []
         if skip_accrued:
             query = "SELECT d.id FROM dividends d LEFT JOIN trades t ON d.account_id=t.account_id "\
-                    "AND d.number=t.number WHERE d.account_id=:account AND t.id IS NULL"
+                    "AND d.asset_id=t.asset_id AND d.number=t.number AND t.number!='' "\
+                    "WHERE d.account_id=:account AND t.id IS NULL"
         else:
             query = "SELECT d.id FROM dividends d WHERE d.account_id=:account"
         params = [(":account", account_id)]
@@ -599,6 +606,9 @@ class Trade(LedgerTransaction):
     def fee(self) -> Decimal:
         return self._fee
 
+    def amount(self) -> Decimal:
+        return self._price * self._qty
+
     def description(self) -> str:
         if self._fee != Decimal('0'):
             text = f"{self._qty:+.2f} @ {self._price:.4f}\n({self._fee:.2f}) "
@@ -861,12 +871,12 @@ class Transfer(LedgerTransaction):
             asset_amount = ledger.getAmount(BookAccount.Assets, self._withdrawal_account.id(), self._asset.id())
             if asset_amount < self._withdrawal:
                 raise ValueError(self.tr("Asset amount is not enough for asset transfer processing. Date: ")
-                                 + f"{ts2str(self._withdrawal_timestamp)}, "
+                                 + f"{ts2dt(self._withdrawal_timestamp)}, "
                                  + f"Asset amount: {asset_amount}, Operation: {self.dump()}")
             processed_qty, processed_value = self._close_deals_fifo(Decimal('-1.0'), self._withdrawal, None)
             if processed_qty < self._withdrawal:
                 raise ValueError(self.tr("Processed asset amount is less than transfer amount. Date: ")
-                                 + f"{ts2str(self._withdrawal_timestamp)}, "
+                                 + f"{ts2dt(self._withdrawal_timestamp)}, "
                                  + f"Processed amount: {asset_amount}, Operation: {self.dump()}")
             if self._withdrawal_currency == JalSettings().getValue('BaseCurrency'):
                 currency_rate = Decimal('1.0')
@@ -1057,11 +1067,11 @@ class CorporateAction(LedgerTransaction):
         asset_amount = ledger.getAmount(BookAccount.Assets, self._account.id(), self._asset.id())
         if asset_amount < self._qty:
             raise ValueError(self.tr("Asset amount is not enough for corporate action processing. Date: ")
-                             + f"{ts2str(self._timestamp)}, "
+                             + f"{ts2dt(self._timestamp)}, "
                              + f"Asset amount: {asset_amount}, Operation: {self.dump()}")
         if asset_amount > self._qty:
             raise ValueError(self.tr("Unhandled case: Corporate action covers not full open position. Date: ")
-                             + f"{ts2str(self._timestamp)}, "
+                             + f"{ts2dt(self._timestamp)}, "
                              + f"Asset amount: {asset_amount}, Operation: {self.dump()}")
         # Calculate total asset allocation after corporate action and verify it equals 100%
         allocation = Decimal('0')
@@ -1070,7 +1080,7 @@ class CorporateAction(LedgerTransaction):
             allocation += Decimal(JalDB.readSQLrecord(query))
         if self._subtype != CorporateAction.Delisting and allocation != Decimal('1.0'):
             raise ValueError(self.tr("Results value of corporate action doesn't match 100% of initial asset value. ")
-                                     + f"Date: {ts2str(self._timestamp)}, "
+                                     + f"Date: {ts2dt(self._timestamp)}, "
                                      + f"Asset amount: {asset_amount}, Operation: {self.dump()}")
         processed_qty, processed_value = self._close_deals_fifo(Decimal('-1.0'), self._qty, None)
         # Withdraw value with old quantity of old asset
