@@ -5,7 +5,7 @@ from PySide6.QtWidgets import QStyledItemDelegate, QHeaderView
 from jal.constants import CustomColor
 from jal.db.ledger import Ledger
 from jal.db.operations import LedgerTransaction
-from jal.widgets.helpers import ts2str
+from jal.widgets.helpers import ts2dt
 
 
 class OperationsModel(QAbstractTableModel):
@@ -16,7 +16,6 @@ class OperationsModel(QAbstractTableModel):
         self._view = parent_view
         self._amount_delegate = None
         self._data = []
-        self._row_count = 0
         self._begin = 0
         self._end = 0
         self._account = 0
@@ -45,9 +44,9 @@ class OperationsModel(QAbstractTableModel):
         row = index.row()
         if not index.isValid():
             return None
-        operation = LedgerTransaction().get_operation(self._data[index.row()]['op_type'],
-                                                      self._data[index.row()]['id'],
-                                                      self._data[index.row()]['subtype'])
+        operation = LedgerTransaction().get_operation(self._data[row]['op_type'],
+                                                      self._data[row]['id'],
+                                                      self._data[row]['subtype'])
         if role == Qt.DisplayRole:
             return self.data_text(operation, index.column())
         if role == Qt.FontRole and index.column() == 0:
@@ -75,7 +74,7 @@ class OperationsModel(QAbstractTableModel):
         if column == 0:
             return operation.label()
         elif column == 1:
-            date_time = ts2str(operation.timestamp())
+            date_time = ts2dt(operation.timestamp())
             if operation.number():
                 date_time += f"\n# {operation.number()}"
             return date_time
@@ -147,10 +146,7 @@ class OperationsModel(QAbstractTableModel):
 
     def prepareData(self):
         self._data = []
-        if self._begin == 0 and self._end == 0:
-            self._row_count = 0
-        else:
-            self._data = Ledger.get_operations_sequence(self._begin, self._end, self._account)
+        self._data = Ledger.get_operations_sequence(self._begin, self._end, self._account)
         self.modelReset.emit()
 
     def deleteRows(self, rows):
