@@ -148,7 +148,11 @@ class FloatDelegate(QStyledItemDelegate):
         editor.setText(localize_decimal(amount, self._tolerance, self._percent))
 
     def setModelData(self, editor, model, index):
-        model.setData(index, delocalize_decimal(editor.text()))
+        number_text = delocalize_decimal(editor.text())
+        value = Decimal(number_text) if number_text else Decimal('0')
+        if self._percent:
+            value /= Decimal('100')
+        model.setData(index, str(value))
 
     def initStyleOption(self, option, index):
         super().initStyleOption(option, index)
