@@ -1,40 +1,14 @@
 from PySide6.QtCore import Qt, QModelIndex
 from PySide6.QtSql import QSqlRelation, QSqlRelationalDelegate, QSqlIndex
-from PySide6.QtWidgets import QToolBar, QAbstractItemView
-from jal.db.helpers import readSQL
+from PySide6.QtWidgets import QAbstractItemView
+from jal.constants import PredefindedAccountType, PredefinedAsset
+from jal.db.db import JalDB
 from jal.db.reference_models import AbstractReferenceListModel, SqlTreeModel
 from jal.widgets.delegates import TimestampDelegate, BoolDelegate, FloatDelegate, \
     PeerSelectorDelegate, AssetSelectorDelegate
 from jal.widgets.reference_data import ReferenceDataDialog
 from jal.widgets.asset_dialog import AssetDialog
 from jal.widgets.delegates import GridLinesDelegate
-from jal.net.downloader import QuoteDownloader
-
-
-# ----------------------------------------------------------------------------------------------------------------------
-class AccountTypeListModel(AbstractReferenceListModel):
-    def __init__(self, table, parent_view):
-        AbstractReferenceListModel.__init__(self, table, parent_view)
-        self._columns = [("id", ''), ("name", self.tr("Account Type"))]
-        self._sort_by = "name"
-        self._hidden = ["id"]
-        self._stretch = "name"
-
-    def configureView(self):
-        super().configureView()
-
-
-class AccountTypeListDialog(ReferenceDataDialog):
-    def __init__(self):
-        ReferenceDataDialog.__init__(self)
-        self.table = "account_types"
-        self.model = AccountTypeListModel(self.table, self.DataView)
-        self.DataView.setModel(self.model)
-        self.model.configureView()
-
-        self.setWindowTitle(self.tr("Account Types"))
-        self.Toggle.setVisible(False)
-        super()._init_completed()
 
 
 # ----------------------------------------------------------------------------------------------------------------------
@@ -49,7 +23,8 @@ class AccountListModel(AbstractReferenceListModel):
                          ("number", self.tr("Account #")),
                          ("reconciled_on", self.tr("Reconciled @")),
                          ("organization_id", self.tr("Bank/Broker")),
-                         ("country_id", self.tr("CC"))]
+                         ("country_id", self.tr("CC")),
+                         ("precision", self.tr("Precision"))]
         self._sort_by = "name"
         self._group_by = "type_id"
         self._hidden = ["id", "type_id"]
@@ -59,7 +34,6 @@ class AccountListModel(AbstractReferenceListModel):
         self._timestamp_delegate = None
         self._bool_delegate = None
         self._default_values = {'active': 1, 'reconciled_on': 0, 'country_id': 0}
-        self.setRelation(self.fieldIndex("type_id"), QSqlRelation("account_types", "id", "name"))
         self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
         self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "code"))
 
@@ -81,14 +55,14 @@ class AccountListModel(AbstractReferenceListModel):
         self._view.setItemDelegateForColumn(self.fieldIndex("active"), self._bool_delegate)
 
     def getAccountType(self, item_id: int) -> int:
-        type_id = readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
+        type_id = JalDB.readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
         type_id = 0 if type_id is None else type_id
         return type_id
 
     def locateItem(self, item_id, use_filter=''):
-        row = readSQL(f"SELECT row_number FROM ("
-                      f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
-                      f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
+        row = JalDB.readSQL(f"SELECT row_number FROM ("
+                             f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
+                             f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
         if row is None:
             return QModelIndex()
         return self.index(row-1, 0)
@@ -115,13 +89,9 @@ class AccountListDialog(ReferenceDataDialog):
         self.GroupLbl.setVisible(True)
         self.GroupLbl.setText(self.tr("Account type:"))
         self.GroupCombo.setVisible(True)
-        self.group_key_field = self.model.group_by
-        self.group_key_index = self.model.fieldIndex(self.model.group_by)
-        self.group_fkey_field = "id"
-        relation_model = self.model.relationModel(self.group_key_index)
-        self.GroupCombo.setModel(relation_model)
-        self.GroupCombo.setModelColumn(relation_model.fieldIndex("name"))
-        self.group_id = relation_model.data(relation_model.index(0, relation_model.fieldIndex(self.group_fkey_field)))
+        self.group_field = self.model.group_by
+        PredefindedAccountType().load2combo(self.GroupCombo)
+        self.group_id = 1
 
     def locateItem(self, item_id):
         type_id = self.model.getAccountType(item_id)
@@ -155,7 +125,6 @@ class AssetListModel(AbstractReferenceListModel):
         self._lookup_delegate = None
         self._timestamp_delegate = None
         self._default_values = {'isin': '', 'country_id': 0, 'quote_source': -1}
-        self.setRelation(self.fieldIndex("type_id"), QSqlRelation("asset_types", "id", "name"))
         self.setRelation(self.fieldIndex("currency_id"), QSqlRelation("currencies", "id", "symbol"))
         self.setRelation(self.fieldIndex("country_id"), QSqlRelation("countries", "id", "name"))
         self.setRelation(self.fieldIndex("quote_source"), QSqlRelation("data_sources", "id", "name"))
@@ -167,14 +136,14 @@ class AssetListModel(AbstractReferenceListModel):
         self._view.setItemDelegateForColumn(self.fieldIndex("quote_source"), self._lookup_delegate)
 
     def getAssetType(self, item_id: int) -> int:
-        type_id = readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
+        type_id = JalDB.readSQL(f"SELECT type_id FROM {self._table} WHERE id=:id", [(":id", item_id)])
         type_id = 0 if type_id is None else type_id
         return type_id
 
     def locateItem(self, item_id, use_filter=''):
-        row = readSQL(f"SELECT row_number FROM ("
-                      f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
-                      f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
+        row = JalDB.readSQL(f"SELECT row_number FROM ("
+                             f"SELECT ROW_NUMBER() OVER (ORDER BY {self._default_name}) AS row_number, id "
+                             f"FROM {self._table} WHERE {use_filter}) WHERE id=:id", [(":id", item_id)])
         if row is None:
             return QModelIndex()
         return self.index(row-1, 0)
@@ -202,20 +171,16 @@ class AssetListDialog(ReferenceDataDialog):
         self.GroupLbl.setVisible(True)
         self.GroupLbl.setText(self.tr("Asset type:"))
         self.GroupCombo.setVisible(True)
-        self.group_key_field = self.model.group_by
-        self.group_key_index = self.model.fieldIndex(self.model.group_by)
-        self.group_fkey_field = "id"
-        self.filter_field = "currency_id"
-        relation_model = self.model.relationModel(self.group_key_index)
-        self.GroupCombo.setModel(relation_model)
-        self.GroupCombo.setModelColumn(relation_model.fieldIndex("name"))
-        self.group_id = relation_model.data(relation_model.index(0, relation_model.fieldIndex(self.group_fkey_field)))
+        self.group_field = self.model.group_by
+        PredefinedAsset().load2combo(self.GroupCombo)
+        self.group_id = 1
 
-        self.toolbar = QToolBar(self)
-        self.search_layout.addWidget(self.toolbar)
-        action = self.toolbar.addAction(self.tr("Update data"))
-        action.setToolTip(self.tr("Update assets data from their exchanges"))
-        action.triggered.connect(self.updateExchangeData)
+        # TODO Probably we need to re-create a button to allow update of asset name/info from exchanges
+        # self.toolbar = QToolBar(self)
+        # self.search_layout.addWidget(self.toolbar)
+        # action = self.toolbar.addAction(self.tr("Update data"))
+        # action.setToolTip(self.tr("Update assets data from their exchanges"))
+        # action.triggered.connect(self.updateExchangeData)
 
     def locateItem(self, item_id):
         type_id = self.model.getAssetType(item_id)
@@ -225,9 +190,6 @@ class AssetListDialog(ReferenceDataDialog):
         item_idx = self.model.locateItem(item_id, use_filter=self._filter_text)
         self.DataView.setCurrentIndex(item_idx)
 
-    def updateExchangeData(self):
-        QuoteDownloader().updataData()
-
     def customEditor(self):
         return AssetDialog()
 
@@ -249,8 +211,8 @@ class PeerTreeModel(SqlTreeModel):
         item_id = index.internalId()
         if role == Qt.DisplayRole:
             if index.column() == 2:
-                return readSQL("SELECT COUNT(d.id) FROM agents AS p "
-                               "LEFT JOIN actions AS d ON d.peer_id=p.id WHERE p.id=:id", [(":id", item_id)])
+                return JalDB.readSQL("SELECT COUNT(d.id) FROM agents AS p "
+                                      "LEFT JOIN actions AS d ON d.peer_id=p.id WHERE p.id=:id", [(":id", item_id)])
             else:
                 return super().data(index, role)
         return None
@@ -337,8 +299,8 @@ class TagListModel(AbstractReferenceListModel):
         self._stretch = "tag"
 
     def locateItem(self, item_id, use_filter=''):
-        row = readSQL(f"SELECT row_number FROM (SELECT ROW_NUMBER() OVER (ORDER BY tag) AS row_number, id "
-                      f"FROM {self._table}) WHERE id=:id", [(":id", item_id)])
+        row = JalDB.readSQL(f"SELECT row_number FROM (SELECT ROW_NUMBER() OVER (ORDER BY tag) AS row_number, id "
+                             f"FROM {self._table}) WHERE id=:id", [(":id", item_id)])
         if row is None:
             return QModelIndex()
         return self.index(row - 1, 0)
